CON
           
' *******************************************************************************************************
' *                                                                                                     *
' *     AiChip I2C Slave Interface                                       AiChip_I2cSlave_Slave_001      *
' *                                                                                                     *
' *******************************************************************************************************
' *                                                                                                     *
' *     Object Name    : AiChip I2C Slave Interface ( Emulates 512 byte Eeprom )                        *
' *     Version        : 001                                                                            *
' *     Created        : 2008-04-11                                                                     *
' *                                                                                                     *
' *     Author         : AiChip Industries                                                              *
' *                                                                                                     *
' *     Copyright      : 2008, AiChip Industries                                                        *
' *                                                                                                     *
' *     Target Chip    : Parallax Propeller Chip P8X32A ( "Propeller Mk I" )                            *
' *     Compiler Used  : Parallax Propeller Tool 1.1                                                    *
' *                                                                                                     *
' *     License        : AiChip Industries and authorised, licensed users only.                         *
' *                                                                                                     *
' *     License        : MIT License. Please see end of this file for Terms of Use.                     *
' *                                                                                                     *
' *******************************************************************************************************
' *                                                                                                     *
' *     AiChip is a trademark of AiChip Industries.                                                     *
' *                                                                                                     *
' *     If distributing a modified version of this software AiChip Industries requests the "AiChip"     *
' *     prefix be removed from the filename and internal text descriptions to avoid any confusion that  *
' *     may be caused by an impression that the modified software originates from AiChip Industries.    *
' *                                                                                                     *
' *******************************************************************************************************

PUB Version

  result := String( "AiChip_I2cSlave_Slave_001" )

CON
      
' .-----------------------------------------------------------------------------------------------------.
' |     Object Handlers                                                                                 |
' `-----------------------------------------------------------------------------------------------------'

CON

  STACK_SIZE = 40
  
VAR

  byte CogNumber

  long stack[100]

PUB Start( sclPin, sdaPin, id, a2, a1, a0 ) | i

  Stop

  id := ( id & %1111_0000 ) | a2 << 3 | a1 << 2 | a0 << 1

  repeat i from 0 to STACK_SIZE-1
    stack[ i ] := $D0D0_F00D
    
  cogNumber := CogNew( I2cSlave( sclPin, sdaPin, id ), @stack ) + 1

PUB Stop

  if cogNumber
    CogStop( cogNumber~ - 1 )

PUB StackUsed

  repeat while stack[ result++ ] <> $D0D0_F00D

CON
      
' .-----------------------------------------------------------------------------------------------------.
' |     Actual I2C Slave Object                                                                         |
' `-----------------------------------------------------------------------------------------------------'

CON

  ADDRESS_MASK = $1FF

VAR
  
  byte bitInputs
  byte bitStream
  
  byte scl
  byte sda
  byte sclShift
  byte sdaShift

  byte deviceId

  byte ram[ ADDRESS_MASK+1 ]

CON
      
' .-----------------------------------------------------------------------------------------------------.
' |     Main I2C Slave Handling Loop                                                                    |
' `-----------------------------------------------------------------------------------------------------'
    
PRI I2cSlave( sclPin, sdaPin, id ) | inputs

  scl := sclPin
  sda := sdaPin

  sclShift := scl-1
  sdaShift := sda

  deviceId := id & $FE
  
  inputs    := INA
  bitInputs := ( ( inputs >> sclShift ) & 2 ) | ( ( inputs >> sdaShift ) & 1 )
  bitStream := ( bitInputs << 4 ) | ( bitInputs << 2 ) | bitInputs
  
  \Idle
  repeat
    \GotStartBit

CON
      
' .-----------------------------------------------------------------------------------------------------.
' |     I2C Command Handling                                                                            |
' `-----------------------------------------------------------------------------------------------------'

PRI GotStartBit | id, adr, num

  ' We only get here when a start bit has been detected
  
  if ( ( id := GetByte ) & $FE ) == deviceId

    ' Command directed to us
    
    if id & 1

      ' Read command
      
      Ack
      repeat
        SendByte( ram[ adr++ & ADDRESS_MASK ] )
      until GetBit

    else

      ' Write Command
      
      Ack
      adr := GetByte << 8
      Ack
      adr |= GetByte
      Ack
      repeat
        num := GetByte
        ram[ adr++ & ADDRESS_MASK ] := num
        Ack

  ' Go into Idle mode - Wait for the next start bit
  
  Idle

PRI Idle

  repeat
    GetBit

CON
      
' .-----------------------------------------------------------------------------------------------------.
' |     Output Routines                                                                                 |
' `-----------------------------------------------------------------------------------------------------'

PRI Ack

  SendBit( 1 )

PRI Nak

  SendBit( 0 )
    
PRI SendByte( n )

  repeat 8
    SendBit( n >> 7 )
    n <<= 1
    
PRI SendBit( n )

  DIRA[ sda ] := ( n & 1 ) ^ 1
  repeat until INA[ scl ]
  repeat while INA[ scl ]
  DIRA[ sda ] := 0

CON
      
' .-----------------------------------------------------------------------------------------------------.
' |     Input Routines                                                                                  |
' `-----------------------------------------------------------------------------------------------------'
  
PRI GetByte

  repeat 8
    result := ( result << 1 ) | GetBit
     
PRI GetBit

  ' Check the bit stream to see if we can detect a sequence which indicates something we can understand,
  '
  ' A start bit will abort which will fall rigt back to the outer loop which will cause the GitStartBit
  ' routine to be executed.
  '
  ' If a stop bit is detected we stop doing anything until we get a subsequent start bit, and handle it
  ' as a start bit, as per above.
  '
  ' Note that the bit stream must have 'clean and identifiable transitions', ie only one line must change
  ' at a time, for example, SDA must be set before SCLK is raised, SCLK must be lowered before SDA is
  ' changed to detect a valid data bit.
  
  repeat
    case GetBitStream
      'CD CD CD
      %11_10_00 : Abort                                                         ' Start Bit
      %00_10_11 : repeat until GetBitStream == %11_10_00                        ' Stop bit
                  Abort                                                         
      %00_10_00 : return 0                                                      ' Bit 0
      %01_11_01 : return 1                                                      ' Bit 1
  
PRI GetBitStream | inputs, thisBits

  ' Build up an incomming bit stream which will have three samples in it of the last three different
  ' states. The bit stream is in the form %CD_CD_CD where C is the SCLK state, D is the SDA state. The
  ' earliest state is on the left, the current state is on the right.
  
  repeat
    inputs   := INA
    thisBits := ( ( inputs >> sclShift ) & 2 ) | ( ( inputs >> sdaShift ) & 1 ) 
  until thisBits <> bitInputs
  
  bitInputs := thisBits

  result := bitStream := ( ( bitStream << 2 ) & %11_11_00 ) | bitInputs

CON

' *******************************************************************************************************
' *                                                                                                     *
' *     Terms of Use : MIT License                                                                      *
' *                                                                                                     *
' *******************************************************************************************************
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
' associated documentation files (the "Software"), to deal in the Software without restriction, including
' without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
' copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to
' the following conditions:                                                                   
'
' The above copyright notice and this permission notice shall be included in all copies or substantial
' portions of the Software.                                                                                             
'                                                                                                                  
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
' LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
' NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
' WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
' SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

CON

' *******************************************************************************************************
' *                                                                                                     *
' *     End of AiChip I2C Slave Interface                                                               *
' *                                                                                                     *
' *******************************************************************************************************   